[Libreoffice-commits] core.git: Branch 'private/mst/sw_redlinehide_3' - 6 commits - sw/inc sw/source
Libreoffice Gerrit user
logerrit at kemper.freedesktop.org
Tue Nov 13 13:01:33 UTC 2018
Rebased ref, commits from common ancestor:
commit 6e1bfb4658191c1498766a9ed7e42161b9caf9f2
Author: Michael Stahl <Michael.Stahl at cib.de>
AuthorDate: Fri Nov 9 16:04:17 2018 +0100
Commit: Michael Stahl <Michael.Stahl at cib.de>
CommitDate: Tue Nov 13 13:57:53 2018 +0100
sw_redlinehide_3: update fields on redline ops
Can't tell which chapter/reference fields will be affected by a redline,
so we have to either update all of them, or rely on the
IDocumentSettingAccess::getFieldUpdateFlags() which lets the SwIdle
update the fields, at some later time...
Change-Id: I3b9cfd17bd227a18f64dac193f0cc912768810c1
diff --git a/sw/source/core/doc/DocumentRedlineManager.cxx b/sw/source/core/doc/DocumentRedlineManager.cxx
index 0d4448a79079..d3f76b3f4ede 100644
--- a/sw/source/core/doc/DocumentRedlineManager.cxx
+++ b/sw/source/core/doc/DocumentRedlineManager.cxx
@@ -22,6 +22,7 @@
#include <txtfrm.hxx>
#include <doc.hxx>
#include <IDocumentUndoRedo.hxx>
+#include <IDocumentFieldsAccess.hxx>
#include <IDocumentState.hxx>
#include <redline.hxx>
#include <UndoRedline.hxx>
@@ -119,6 +120,13 @@ using namespace com::sun::star;
namespace sw {
+static void UpdateFieldsForRedline(IDocumentFieldsAccess & rIDFA)
+{
+ rIDFA.GetSysFieldType(SwFieldIds::Chapter)->UpdateFields();
+ rIDFA.UpdateExpFields(nullptr, false);
+ rIDFA.UpdateRefFields();
+}
+
void UpdateFramesForAddDeleteRedline(SwDoc & rDoc, SwPaM const& rPam)
{
// no need to call UpdateFootnoteNums for FTNNUM_PAGE:
@@ -148,6 +156,8 @@ void UpdateFramesForAddDeleteRedline(SwDoc & rDoc, SwPaM const& rPam)
// node of the merged frame, there could be another redline nearby
sw::AddRemoveFlysAnchoredToFrameStartingAtNode(*pFrame, *pStartNode, nullptr);
}
+ // fields last - SwGetRefField::UpdateField requires up-to-date frames
+ UpdateFieldsForRedline(rDoc.getIDocumentFieldsAccess()); // after footnotes
}
void UpdateFramesForRemoveDeleteRedline(SwDoc & rDoc, SwPaM const& rPam)
@@ -205,6 +215,8 @@ void UpdateFramesForRemoveDeleteRedline(SwDoc & rDoc, SwPaM const& rPam)
AppendAllObjs(rDoc.GetSpzFrameFormats(), pLayout);
}
}
+ // fields last - SwGetRefField::UpdateField requires up-to-date frames
+ UpdateFieldsForRedline(rDoc.getIDocumentFieldsAccess()); // after footnotes
}
} // namespace sw
commit a6ce3f9d6bf7224127f16439fdd047476ad2da67
Author: Michael Stahl <Michael.Stahl at cib.de>
AuthorDate: Fri Nov 9 12:50:04 2018 +0100
Commit: Michael Stahl <Michael.Stahl at cib.de>
CommitDate: Tue Nov 13 13:57:53 2018 +0100
sw_redlinehide_3: update fields in UnHideRedlines
Repainting is not enough, values require up-to-date outline visibility
etc.
Change-Id: I85d99fc65a071279c2d4656a52ff82ed3b2db7d8
diff --git a/sw/source/core/layout/wsfrm.cxx b/sw/source/core/layout/wsfrm.cxx
index eb8a0fa12368..fd3ab2b10dd0 100644
--- a/sw/source/core/layout/wsfrm.cxx
+++ b/sw/source/core/layout/wsfrm.cxx
@@ -4504,6 +4504,10 @@ void SwRootFrame::SetHideRedlines(bool const bHideRedlines)
pFootnote->InvalidateNumberInLayout();
}
}
+ // update various fields to re-expand them with the new layout
+ rDoc.getIDocumentFieldsAccess().GetSysFieldType(SwFieldIds::Chapter)->UpdateFields();
+ rDoc.getIDocumentFieldsAccess().UpdateExpFields(nullptr, false);
+ rDoc.getIDocumentFieldsAccess().UpdateRefFields();
// InvalidateAllContent(SwInvalidateFlags::Size); // ??? TODO what to invalidate? this is the big hammer
}
commit 95c2712a763aa8b29638c93d189ddf5eae4c1bf7
Author: Michael Stahl <Michael.Stahl at cib.de>
AuthorDate: Tue Nov 13 11:00:33 2018 +0100
Commit: Michael Stahl <Michael.Stahl at cib.de>
CommitDate: Tue Nov 13 13:55:52 2018 +0100
sw_redlinehide_3: add second result to SwGetExpField, SwSetExpField
This is quite tricky & somewhat ugly due to the SwValueField base class
& we need to duplicate its member too :(
It would be possible to filter the deleted fields in
SwDocUpdateField::MakeFieldList(), but the problem with that is that
the instance is long-lived & might be used with different layout
settings, so it's probably better to filter in the 5 or so client
functions that iterate over the GetSortList().
Change-Id: I0cdbc7757fa529598b5dbceec0bd2c4a619be05a
diff --git a/sw/inc/IDocumentFieldsAccess.hxx b/sw/inc/IDocumentFieldsAccess.hxx
index 5b8299ac14db..617ea990614c 100644
--- a/sw/inc/IDocumentFieldsAccess.hxx
+++ b/sw/inc/IDocumentFieldsAccess.hxx
@@ -38,6 +38,7 @@ class SwNode;
enum class SwFieldIds : sal_uInt16;
template <class T> class SwHashTable;
struct HashStr;
+class SwRootFrame;
namespace rtl { class OUString; }
using rtl::OUString;
@@ -122,9 +123,9 @@ namespace com { namespace sun { namespace star { namespace uno { class Any; } }
// (if the address != 0 and the pointer == 0 a new list will be returned).
virtual void FieldsToCalc(SwCalc& rCalc, sal_uLong nLastNd, sal_uInt16 nLastCnt) = 0;
- virtual void FieldsToCalc(SwCalc& rCalc, const SetGetExpField& rToThisField) = 0;
+ virtual void FieldsToCalc(SwCalc& rCalc, const SetGetExpField& rToThisField, SwRootFrame const* pLayout) = 0;
- virtual void FieldsToExpand(SwHashTable<HashStr> & rTable, const SetGetExpField& rToThisField) = 0;
+ virtual void FieldsToExpand(SwHashTable<HashStr> & rTable, const SetGetExpField& rToThisField, SwRootFrame const& rLayout) = 0;
virtual bool IsNewFieldLst() const = 0;
diff --git a/sw/inc/cellfml.hxx b/sw/inc/cellfml.hxx
index 96b52b576743..f9fd74c09c57 100644
--- a/sw/inc/cellfml.hxx
+++ b/sw/inc/cellfml.hxx
@@ -31,6 +31,7 @@ class SwCalc;
class SwTableBox;
class SwTableFormulaUpdate;
class SwDoc;
+class SwRootFrame;
class SwTableCalcPara
{
@@ -38,11 +39,12 @@ class SwTableCalcPara
sal_uInt16 m_nStackCount, m_nMaxSize;
public:
+ SwRootFrame const*const m_pLayout; ///< layout to access text field results
std::unique_ptr<SwTableSortBoxes> m_pBoxStack; ///< stack for recognizing recursion
SwCalc& m_rCalc; ///< current Calculator
const SwTable* m_pTable; ///< current table
- SwTableCalcPara( SwCalc& rCalculator, const SwTable& rTable );
+ SwTableCalcPara(SwCalc& rCalculator, const SwTable& rTable, SwRootFrame const* pLayout);
~SwTableCalcPara();
bool CalcWithStackOverflow();
diff --git a/sw/inc/expfld.hxx b/sw/inc/expfld.hxx
index 7ec48a69af5d..c82d1e0edab8 100644
--- a/sw/inc/expfld.hxx
+++ b/sw/inc/expfld.hxx
@@ -80,7 +80,9 @@ protected:
class SW_DLLPUBLIC SwGetExpField : public SwFormulaField
{
+ double m_fValueRLHidden; ///< SwValueField; hidden redlines
OUString m_sExpand;
+ OUString m_sExpandRLHidden; ///< hidden redlines
bool m_bIsInBodyText;
sal_uInt16 m_nSubType;
@@ -88,15 +90,19 @@ class SW_DLLPUBLIC SwGetExpField : public SwFormulaField
virtual OUString ExpandImpl(SwRootFrame const* pLayout) const override;
virtual std::unique_ptr<SwField> Copy() const override;
+ using SwFormulaField::GetValue; // hide it, don't use
+ virtual void SetValue(const double& rVal) override; // hide it
public:
SwGetExpField( SwGetExpFieldType*, const OUString& rFormel,
sal_uInt16 nSubType, sal_uLong nFormat);
- virtual void SetValue( const double& rVal ) override;
+ double GetValue(SwRootFrame const* pLayout) const;
+ void SetValue(const double& rVal, SwRootFrame const* pLayout);
+
virtual void SetLanguage(LanguageType nLng) override;
- inline void ChgExpStr(const OUString& rExpand);
+ void ChgExpStr(const OUString& rExpand, SwRootFrame const* pLayout);
/// Called by formatting.
inline bool IsInBodyText() const;
@@ -124,9 +130,6 @@ public:
void SetLateInitialization() { m_bLateInitialization = true;}
};
-inline void SwGetExpField::ChgExpStr(const OUString& rExpand)
- { m_sExpand = rExpand;}
-
/// Called by formatting.
inline bool SwGetExpField::IsInBodyText() const
{ return m_bIsInBodyText; }
@@ -175,7 +178,7 @@ public:
void SetDelimiter( const OUString& s ) { m_sDelim = s; }
sal_uInt8 GetOutlineLvl() const { return m_nLevel; }
void SetOutlineLvl( sal_uInt8 n ) { m_nLevel = n; }
- void SetChapter( SwSetExpField& rField, const SwNode& rNd );
+ void SetChapter(SwSetExpField& rField, const SwNode& rNd, SwRootFrame const* pLayout);
virtual void QueryValue( css::uno::Any& rVal, sal_uInt16 nWhich ) const override;
virtual void PutValue( const css::uno::Any& rVal, sal_uInt16 nWhich ) override;
@@ -195,7 +198,9 @@ inline const OUString& SwSetExpFieldType::GetSetRefName() const
class SW_DLLPUBLIC SwSetExpField : public SwFormulaField
{
+ double m_fValueRLHidden; ///< SwValueField; hidden redlines
OUString msExpand;
+ OUString msExpandRLHidden; ///< hidden redlines
OUString maPText;
bool mbInput;
sal_uInt16 mnSeqNo;
@@ -204,6 +209,8 @@ class SW_DLLPUBLIC SwSetExpField : public SwFormulaField
virtual OUString ExpandImpl(SwRootFrame const* pLayout) const override;
virtual std::unique_ptr<SwField> Copy() const override;
+ using SwFormulaField::GetValue; // hide it, don't use
+ virtual void SetValue(const double& rVal) override; // hide it
public:
SwSetExpField(SwSetExpFieldType*, const OUString& rFormel, sal_uLong nFormat = 0);
@@ -211,11 +218,12 @@ public:
void SetFormatField(SwFormatField & rFormatField);
SwFormatField* GetFormatField() { return mpFormatField;}
- virtual void SetValue( const double& rVal ) override;
+ double GetValue(SwRootFrame const* pLayout) const;
+ void SetValue(const double& rVal, SwRootFrame const* pLayout);
- inline const OUString& GetExpStr() const;
+ const OUString& GetExpStr(SwRootFrame const* pLayout) const;
- inline void ChgExpStr( const OUString& rExpand );
+ void ChgExpStr(const OUString& rExpand, SwRootFrame const* pLayout);
inline void SetPromptText(const OUString& rStr);
inline const OUString& GetPromptText() const;
@@ -244,12 +252,6 @@ public:
virtual bool PutValue( const css::uno::Any& rVal, sal_uInt16 nWhich ) override;
};
-inline const OUString& SwSetExpField::GetExpStr() const
- { return msExpand; }
-
-inline void SwSetExpField::ChgExpStr( const OUString& rExpand )
- { msExpand = rExpand; }
-
inline void SwSetExpField::SetPromptText(const OUString& rStr)
{ maPText = rStr; }
diff --git a/sw/source/core/doc/DocumentFieldsManager.cxx b/sw/source/core/doc/DocumentFieldsManager.cxx
index f24fd9688238..21bccdebcf6c 100644
--- a/sw/source/core/doc/DocumentFieldsManager.cxx
+++ b/sw/source/core/doc/DocumentFieldsManager.cxx
@@ -22,6 +22,9 @@
#include <IDocumentUndoRedo.hxx>
#include <IDocumentState.hxx>
#include <IDocumentLayoutAccess.hxx>
+#include <IDocumentRedlineAccess.hxx>
+#include <redline.hxx>
+#include <rootfrm.hxx>
#include <dbmgr.hxx>
#include <chpfld.hxx>
#include <dbfld.hxx>
@@ -78,13 +81,41 @@ namespace
#endif
+ bool IsFieldDeleted(IDocumentRedlineAccess const& rIDRA,
+ SwRootFrame const& rLayout, SwTextField const& rTextField)
+ {
+ SwTextNode const& rNode(rTextField.GetTextNode());
+ bool const isInBody(
+ rNode.GetNodes().GetEndOfExtras().GetIndex() < rNode.GetIndex());
+ if (!isInBody && nullptr == rNode.getLayoutFrame(&rLayout))
+ { // see SwDocUpdateField::GetBodyNode() - fields in hidden sections
+ // don't have layout frames but must be updated, so use the same
+ // check as there, but do it again because GetBodyNode() checks
+ // for *any* layout...
+ return true;
+ }
+ SwRedlineTable::size_type tmp;
+ SwPosition const pos(const_cast<SwTextNode&>(rTextField.GetTextNode()),
+ rTextField.GetStart());
+ SwRangeRedline const*const pRedline(rIDRA.GetRedline(pos, &tmp));
+ return (pRedline
+ && pRedline->GetType() == nsRedlineType_t::REDLINE_DELETE
+ && *pRedline->GetPoint() != *pRedline->GetMark());
+ }
+
void lcl_CalcField( SwDoc& rDoc, SwCalc& rCalc, const SetGetExpField& rSGEField,
- SwDBManager* pMgr )
+ SwDBManager* pMgr, SwRootFrame const*const pLayout)
{
const SwTextField* pTextField = rSGEField.GetTextField();
if( !pTextField )
return ;
+ if (pLayout && pLayout->IsHideRedlines()
+ && IsFieldDeleted(rDoc.getIDocumentRedlineAccess(), *pLayout, *pTextField))
+ {
+ return;
+ }
+
const SwField* pField = pTextField->GetFormatField().GetField();
const SwFieldIds nFieldWhich = pField->GetTyp()->Which();
@@ -92,10 +123,10 @@ namespace
{
SwSbxValue aValue;
if( nsSwGetSetExpType::GSE_EXPR & pField->GetSubType() )
- aValue.PutDouble( static_cast<const SwSetExpField*>(pField)->GetValue() );
+ aValue.PutDouble( static_cast<const SwSetExpField*>(pField)->GetValue(pLayout) );
else
// Extension to calculate with Strings
- aValue.PutString( static_cast<const SwSetExpField*>(pField)->GetExpStr() );
+ aValue.PutString( static_cast<const SwSetExpField*>(pField)->GetExpStr(pLayout) );
// set the new value in Calculator
rCalc.VarChange( pField->GetTyp()->GetName(), aValue );
@@ -666,6 +697,13 @@ void DocumentFieldsManager::UpdateTableFields( SfxPoolItem* pHt )
}
}
+ SwRootFrame const* pLayout(nullptr);
+ for (SwRootFrame const*const pLay : m_rDoc.GetAllLayouts())
+ {
+ assert(!pLayout || pLay->IsHideRedlines() == pLayout->IsHideRedlines()); // TODO
+ pLayout = pLay;
+ }
+
// all fields/boxes are now invalid, so we can start to calculate
if( pHt && ( RES_TABLEFML_UPDATE != pHt->Which() ||
TBL_CALC != static_cast<SwTableFormulaUpdate*>(pHt)->m_eFlags ))
@@ -724,7 +762,7 @@ void DocumentFieldsManager::UpdateTableFields( SfxPoolItem* pHt )
if( GetBodyTextNode( m_rDoc, aPos, *pFrame ) )
FieldsToCalc( *pCalc, SetGetExpField(
aPos.nNode, pFormatField->GetTextField(),
- &aPos.nContent ));
+ &aPos.nContent), pLayout);
else
pFrame = nullptr;
}
@@ -734,10 +772,11 @@ void DocumentFieldsManager::UpdateTableFields( SfxPoolItem* pHt )
// create index to determine the TextNode
SwNodeIndex aIdx( rTextNd );
FieldsToCalc( *pCalc,
- SetGetExpField( aIdx, pFormatField->GetTextField() ));
+ SetGetExpField(aIdx, pFormatField->GetTextField()),
+ pLayout);
}
- SwTableCalcPara aPara( *pCalc, pTableNd->GetTable() );
+ SwTableCalcPara aPara(*pCalc, pTableNd->GetTable(), pLayout);
pField->CalcField( aPara );
if( aPara.IsStackOverflow() )
{
@@ -794,7 +833,8 @@ void DocumentFieldsManager::UpdateTableFields( SfxPoolItem* pHt )
{
SwPosition aPos( *pCNd );
if( GetBodyTextNode( m_rDoc, aPos, *pFrame ) )
- FieldsToCalc( *pCalc, SetGetExpField( aPos.nNode ));
+ FieldsToCalc(*pCalc, SetGetExpField(aPos.nNode),
+ pLayout);
else
pFrame = nullptr;
}
@@ -803,10 +843,10 @@ void DocumentFieldsManager::UpdateTableFields( SfxPoolItem* pHt )
{
// create index to determine the TextNode
SwNodeIndex aIdx( *pTableNd );
- FieldsToCalc( *pCalc, SetGetExpField( aIdx ));
+ FieldsToCalc(*pCalc, SetGetExpField(aIdx), pLayout);
}
- SwTableCalcPara aPara( *pCalc, pTableNd->GetTable() );
+ SwTableCalcPara aPara(*pCalc, pTableNd->GetTable(), pLayout);
pFormula->Calc( aPara, nValue );
if( aPara.IsStackOverflow() )
@@ -859,6 +899,39 @@ void DocumentFieldsManager::UpdateExpFields( SwTextField* pUpdateField, bool bUp
return ;
}
+ SwRootFrame const* pLayout(nullptr);
+ SwRootFrame const* pLayoutRLHidden(nullptr);
+ for (SwRootFrame const*const pLay : m_rDoc.GetAllLayouts())
+ {
+ if (pLay->IsHideRedlines())
+ {
+ pLayoutRLHidden = pLay;
+ }
+ else
+ {
+ pLayout = pLay;
+ }
+ }
+ if (pLayout || !pLayoutRLHidden) // always calc *something*...
+ {
+ UpdateExpFieldsImpl(pUpdateField, pLayout);
+ }
+ if (pLayoutRLHidden)
+ {
+ UpdateExpFieldsImpl(pUpdateField, pLayoutRLHidden);
+ }
+
+ // update reference fields
+ if( bUpdRefFields )
+ UpdateRefFields();
+
+ mpUpdateFields->SetInUpdateFields( bOldInUpdateFields );
+ mpUpdateFields->SetFieldsDirty( false );
+}
+
+void DocumentFieldsManager::UpdateExpFieldsImpl(
+ SwTextField * pUpdateField, SwRootFrame const*const pLayout)
+{
SwFieldIds nWhich;
// Hash table for all string replacements is filled on-the-fly.
@@ -925,6 +998,7 @@ void DocumentFieldsManager::UpdateExpFields( SwTextField* pUpdateField, bool bUp
}
}
+ IDocumentRedlineAccess const& rIDRA(m_rDoc.getIDocumentRedlineAccess());
std::unordered_map<SwSetExpFieldType const*, SwTextNode const*> SetExpOutlineNodeMap;
for (std::unique_ptr<SetGetExpField> const& it : *mpUpdateFields->GetSortList())
@@ -967,6 +1041,12 @@ void DocumentFieldsManager::UpdateExpFields( SwTextField* pUpdateField, bool bUp
continue;
}
+ if (pLayout && pLayout->IsHideRedlines()
+ && IsFieldDeleted(rIDRA, *pLayout, *pTextField))
+ {
+ continue;
+ }
+
SwFormatField* pFormatField = const_cast<SwFormatField*>(&pTextField->GetFormatField());
const SwField* pField = pFormatField->GetField();
@@ -1058,7 +1138,7 @@ void DocumentFieldsManager::UpdateExpFields( SwTextField* pUpdateField, bool bUp
&& pGField->IsInBodyText() )
{
OUString aNew = LookString( aHashStrTable, pGField->GetFormula() );
- pGField->ChgExpStr( aNew );
+ pGField->ChgExpStr( aNew, pLayout );
}
}
else
@@ -1072,7 +1152,7 @@ void DocumentFieldsManager::UpdateExpFields( SwTextField* pUpdateField, bool bUp
// only update one field
if( !pUpdateField || pUpdateField == pTextField )
- pSField->ChgExpStr( aNew );
+ pSField->ChgExpStr( aNew, pLayout );
// lookup the field's name
aNew = static_cast<SwSetExpFieldType*>(pSField->GetTyp())->GetSetRefName();
@@ -1081,12 +1161,12 @@ void DocumentFieldsManager::UpdateExpFields( SwTextField* pUpdateField, bool bUp
HashStr* pFnd = aHashStrTable.Find( aNew, &nPos );
if( pFnd )
// Modify entry in the hash table
- pFnd->aSetStr = pSField->GetExpStr();
+ pFnd->aSetStr = pSField->GetExpStr(pLayout);
else
{
// insert new entry
aHashStrTable[nPos].reset( new HashStr( aNew,
- pSField->GetExpStr(),
+ pSField->GetExpStr(pLayout),
aHashStrTable[nPos].release() ) );
pFnd = aHashStrTable[nPos].get();
}
@@ -1109,7 +1189,7 @@ void DocumentFieldsManager::UpdateExpFields( SwTextField* pUpdateField, bool bUp
SwSbxValue aValue = aCalc.Calculate(
pGField->GetFormula());
if(!aValue.IsVoidValue())
- pGField->SetValue(aValue.GetDouble() );
+ pGField->SetValue(aValue.GetDouble(), pLayout);
}
}
else
@@ -1129,7 +1209,7 @@ void DocumentFieldsManager::UpdateExpFields( SwTextField* pUpdateField, bool bUp
pSeqNd = m_rDoc.GetNodes()[ it->GetNode() ];
const SwTextNode* pOutlNd = pSeqNd->
- FindOutlineNodeOfLevel( nLvl );
+ FindOutlineNodeOfLevel(nLvl, pLayout);
auto const iter(SetExpOutlineNodeMap.find(pSFieldTyp));
if (iter == SetExpOutlineNodeMap.end()
|| iter->second != pOutlNd)
@@ -1148,10 +1228,10 @@ void DocumentFieldsManager::UpdateExpFields( SwTextField* pUpdateField, bool bUp
// only update one field
if( !aValue.IsVoidValue() && (!pUpdateField || pUpdateField == pTextField) )
{
- pSField->SetValue( nErg );
+ pSField->SetValue(nErg, pLayout);
if( pSeqNd )
- pSFieldTyp->SetChapter( *pSField, *pSeqNd );
+ pSFieldTyp->SetChapter(*pSField, *pSeqNd, pLayout);
}
}
}
@@ -1175,13 +1255,6 @@ void DocumentFieldsManager::UpdateExpFields( SwTextField* pUpdateField, bool bUp
#if HAVE_FEATURE_DBCONNECTIVITY
pMgr->CloseAll(false);
#endif
-
- // update reference fields
- if( bUpdRefFields )
- UpdateRefFields();
-
- mpUpdateFields->SetInUpdateFields( bOldInUpdateFields );
- mpUpdateFields->SetFieldsDirty( false );
}
/// Insert field type that was marked as deleted
@@ -1429,7 +1502,8 @@ void DocumentFieldsManager::SetFixFields( const DateTime* pNewDateTime )
m_rDoc.getIDocumentState().ResetModified();
}
-void DocumentFieldsManager::FieldsToCalc( SwCalc& rCalc, const SetGetExpField& rToThisField )
+void DocumentFieldsManager::FieldsToCalc(SwCalc& rCalc,
+ const SetGetExpField& rToThisField, SwRootFrame const*const pLayout)
{
// create the sorted list of all SetFields
mpUpdateFields->MakeFieldList( m_rDoc, mbNewFieldLst, GETFLD_CALC );
@@ -1449,7 +1523,7 @@ void DocumentFieldsManager::FieldsToCalc( SwCalc& rCalc, const SetGetExpField& r
&rToThisField);
for (auto it = mpUpdateFields->GetSortList()->begin(); it != itLast; ++it)
{
- lcl_CalcField( m_rDoc, rCalc, **it, pMgr );
+ lcl_CalcField(m_rDoc, rCalc, **it, pMgr, pLayout);
}
}
#if HAVE_FEATURE_DBCONNECTIVITY
@@ -1470,6 +1544,22 @@ void DocumentFieldsManager::FieldsToCalc( SwCalc& rCalc, sal_uLong nLastNd, sal_
pMgr->CloseAll(false);
#endif
+ SwRootFrame const* pLayout(nullptr);
+ SwRootFrame const* pLayoutRLHidden(nullptr);
+ for (SwRootFrame const*const pLay : m_rDoc.GetAllLayouts())
+ {
+ if (pLay->IsHideRedlines())
+ {
+ pLayoutRLHidden = pLay;
+ }
+ else
+ {
+ pLayout = pLay;
+ }
+ }
+
+ // note this is not duplicate of the other FieldsToCalc because there is
+ // (currently) no SetGetExpField that compares only a position
for(auto it = mpUpdateFields->GetSortList()->begin();
it != mpUpdateFields->GetSortList()->end() &&
( (*it)->GetNode() < nLastNd ||
@@ -1477,7 +1567,14 @@ void DocumentFieldsManager::FieldsToCalc( SwCalc& rCalc, sal_uLong nLastNd, sal_
);
++it )
{
- lcl_CalcField( m_rDoc, rCalc, **it, pMgr );
+ if (pLayout || !pLayoutRLHidden) // always calc *something*...
+ {
+ lcl_CalcField( m_rDoc, rCalc, **it, pMgr, pLayout );
+ }
+ if (pLayoutRLHidden)
+ {
+ lcl_CalcField( m_rDoc, rCalc, **it, pMgr, pLayoutRLHidden );
+ }
}
#if HAVE_FEATURE_DBCONNECTIVITY
@@ -1486,12 +1583,14 @@ void DocumentFieldsManager::FieldsToCalc( SwCalc& rCalc, sal_uLong nLastNd, sal_
}
void DocumentFieldsManager::FieldsToExpand( SwHashTable<HashStr> & rHashTable,
- const SetGetExpField& rToThisField )
+ const SetGetExpField& rToThisField, SwRootFrame const& rLayout)
{
// create the sorted list of all SetFields
mpUpdateFields->MakeFieldList( m_rDoc, mbNewFieldLst, GETFLD_EXPAND );
mbNewFieldLst = false;
+ IDocumentRedlineAccess const& rIDRA(m_rDoc.getIDocumentRedlineAccess());
+
// Hash table for all string replacements is filled on-the-fly.
// Try to fabricate an uneven number.
sal_uInt16 nTableSize = ((mpUpdateFields->GetSortList()->size() / 7) + 1) * 7;
@@ -1506,6 +1605,12 @@ void DocumentFieldsManager::FieldsToExpand( SwHashTable<HashStr> & rHashTable,
if( !pTextField )
continue;
+ if (rLayout.IsHideRedlines()
+ && IsFieldDeleted(rIDRA, rLayout, *pTextField))
+ {
+ continue;
+ }
+
const SwField* pField = pTextField->GetFormatField().GetField();
switch( pField->GetTyp()->Which() )
{
@@ -1522,7 +1627,7 @@ void DocumentFieldsManager::FieldsToExpand( SwHashTable<HashStr> & rHashTable,
// #i3141# - update expression of field as in method
// <SwDoc::UpdateExpFields(..)> for string/text fields
- pSField->ChgExpStr( aNew );
+ pSField->ChgExpStr(aNew, &rLayout);
// look up the field's name
aNew = static_cast<SwSetExpFieldType*>(pSField->GetTyp())->GetSetRefName();
@@ -1531,11 +1636,11 @@ void DocumentFieldsManager::FieldsToExpand( SwHashTable<HashStr> & rHashTable,
SwHash* pFnd = rHashTable.Find( aNew, &nPos );
if( pFnd )
// modify entry in the hash table
- static_cast<HashStr*>(pFnd)->aSetStr = pSField->GetExpStr();
+ static_cast<HashStr*>(pFnd)->aSetStr = pSField->GetExpStr(&rLayout);
else
// insert the new entry
rHashTable[nPos].reset( new HashStr( aNew,
- pSField->GetExpStr(), rHashTable[nPos].release() ) );
+ pSField->GetExpStr(&rLayout), rHashTable[nPos].release()));
}
break;
case SwFieldIds::Database:
diff --git a/sw/source/core/fields/cellfml.cxx b/sw/source/core/fields/cellfml.cxx
index 836ae5c24ef6..2edf4762c3e4 100644
--- a/sw/source/core/fields/cellfml.cxx
+++ b/sw/source/core/fields/cellfml.cxx
@@ -163,7 +163,7 @@ double SwTableBox::GetValue( SwTableCalcPara& rCalcPara ) const
switch ( pField->GetTyp()->Which() )
{
case SwFieldIds::SetExp:
- nRet = static_cast<const SwSetExpField*>(pField)->GetValue();
+ nRet = static_cast<const SwSetExpField*>(pField)->GetValue(rCalcPara.m_pLayout);
break;
case SwFieldIds::User:
nRet = static_cast<const SwUserField*>(pField)->GetValue();
@@ -251,10 +251,15 @@ double SwTableBox::GetValue( SwTableCalcPara& rCalcPara ) const
// structure needed for calculation of tables
-SwTableCalcPara::SwTableCalcPara( SwCalc& rCalculator, const SwTable& rTable )
- : m_pLastTableBox( nullptr ), m_nStackCount( 0 ), m_nMaxSize( cMAXSTACKSIZE ),
- m_pBoxStack( new SwTableSortBoxes ),
- m_rCalc( rCalculator ), m_pTable( &rTable )
+SwTableCalcPara::SwTableCalcPara(SwCalc& rCalculator, const SwTable& rTable,
+ SwRootFrame const*const pLayout)
+ : m_pLastTableBox(nullptr)
+ , m_nStackCount( 0 )
+ , m_nMaxSize( cMAXSTACKSIZE )
+ , m_pLayout(pLayout)
+ , m_pBoxStack( new SwTableSortBoxes )
+ , m_rCalc( rCalculator )
+ , m_pTable( &rTable )
{
}
diff --git a/sw/source/core/fields/expfld.cxx b/sw/source/core/fields/expfld.cxx
index 9884be99aee3..dc7706c1652b 100644
--- a/sw/source/core/fields/expfld.cxx
+++ b/sw/source/core/fields/expfld.cxx
@@ -283,7 +283,9 @@ void SwGetExpFieldType::Modify( const SfxPoolItem*, const SfxPoolItem* pNew )
SwGetExpField::SwGetExpField(SwGetExpFieldType* pTyp, const OUString& rFormel,
sal_uInt16 nSub, sal_uLong nFormat)
- : SwFormulaField( pTyp, nFormat, 0.0 ),
+ : SwFormulaField( pTyp, nFormat, 0.0 )
+ , m_fValueRLHidden(0.0)
+ ,
m_bIsInBodyText( true ),
m_nSubType(nSub),
m_bLateInitialization( false )
@@ -291,12 +293,24 @@ SwGetExpField::SwGetExpField(SwGetExpFieldType* pTyp, const OUString& rFormel,
SetFormula( rFormel );
}
-OUString SwGetExpField::ExpandImpl(SwRootFrame const*const) const
+void SwGetExpField::ChgExpStr(const OUString& rExpand, SwRootFrame const*const pLayout)
+{
+ if (!pLayout || pLayout->IsHideRedlines())
+ {
+ m_sExpandRLHidden = rExpand;
+ }
+ if (!pLayout || !pLayout->IsHideRedlines())
+ {
+ m_sExpand = rExpand;
+ }
+}
+
+OUString SwGetExpField::ExpandImpl(SwRootFrame const*const pLayout) const
{
if(m_nSubType & nsSwExtendedSubType::SUB_CMD)
return GetFormula();
- return m_sExpand;
+ return (pLayout && pLayout->IsHideRedlines()) ? m_sExpandRLHidden : m_sExpand;
}
OUString SwGetExpField::GetFieldName() const
@@ -314,6 +328,7 @@ std::unique_ptr<SwField> SwGetExpField::Copy() const
std::unique_ptr<SwGetExpField> pTmp(new SwGetExpField(static_cast<SwGetExpFieldType*>(GetTyp()),
GetFormula(), m_nSubType, GetFormat()));
pTmp->SetLanguage(GetLanguage());
+ pTmp->m_fValueRLHidden = m_fValueRLHidden;
pTmp->SwValueField::SetValue(GetValue());
pTmp->m_sExpand = m_sExpand;
pTmp->m_bIsInBodyText = m_bIsInBodyText;
@@ -356,25 +371,27 @@ void SwGetExpField::ChangeExpansion( const SwFrame& rFrame, const SwTextField& r
}
}
+ SwRootFrame const& rLayout(*rFrame.getRootFrame());
+ OUString & rExpand(rLayout.IsHideRedlines() ? m_sExpandRLHidden : m_sExpand);
SetGetExpField aEndField( aPos.nNode, &rField, &aPos.nContent );
if(GetSubType() & nsSwGetSetExpType::GSE_STRING)
{
SwHashTable<HashStr> aHashTable(0);
- rDoc.getIDocumentFieldsAccess().FieldsToExpand( aHashTable, aEndField );
- m_sExpand = LookString( aHashTable, GetFormula() );
+ rDoc.getIDocumentFieldsAccess().FieldsToExpand(aHashTable, aEndField, rLayout);
+ rExpand = LookString( aHashTable, GetFormula() );
}
else
{
// fill calculator with values
SwCalc aCalc( rDoc );
- rDoc.getIDocumentFieldsAccess().FieldsToCalc(aCalc, aEndField);
+ rDoc.getIDocumentFieldsAccess().FieldsToCalc(aCalc, aEndField, &rLayout);
// calculate value
- SetValue(aCalc.Calculate(GetFormula()).GetDouble());
+ SetValue(aCalc.Calculate(GetFormula()).GetDouble(), &rLayout);
// analyse based on format
- m_sExpand = static_cast<SwValueFieldType*>(GetTyp())->ExpandValue(
- GetValue(), GetFormat(), GetLanguage());
+ rExpand = static_cast<SwValueFieldType*>(GetTyp())->ExpandValue(
+ GetValue(&rLayout), GetFormat(), GetLanguage());
}
}
@@ -447,6 +464,7 @@ bool SwGetExpField::PutValue( const uno::Any& rAny, sal_uInt16 nWhichId )
{
case FIELD_PROP_DOUBLE:
SwValueField::SetValue(*o3tl::doAccess<double>(rAny));
+ m_fValueRLHidden = *o3tl::doAccess<double>(rAny);
break;
case FIELD_PROP_FORMAT:
rAny >>= nTmp;
@@ -478,7 +496,7 @@ bool SwGetExpField::PutValue( const uno::Any& rAny, sal_uInt16 nWhichId )
{
OUString sTmp;
rAny >>= sTmp;
- ChgExpStr(sTmp);
+ ChgExpStr(sTmp, nullptr);
break;
}
default:
@@ -513,6 +531,23 @@ OUString SwSetExpFieldType::GetName() const
return m_sName;
}
+const OUString& SwSetExpField::GetExpStr(SwRootFrame const*const pLayout) const
+{
+ return (pLayout && pLayout->IsHideRedlines()) ? msExpandRLHidden : msExpand;
+}
+
+void SwSetExpField::ChgExpStr(const OUString& rExpand, SwRootFrame const*const pLayout)
+{
+ if (!pLayout || pLayout->IsHideRedlines())
+ {
+ msExpandRLHidden = rExpand;
+ }
+ if (!pLayout || !pLayout->IsHideRedlines())
+ {
+ msExpand = rExpand;
+ }
+}
+
void SwSetExpFieldType::Modify( const SfxPoolItem*, const SfxPoolItem* )
{
// do not expand further
@@ -605,9 +640,10 @@ size_t SwSetExpFieldType::GetSeqFieldList(SwSeqFieldList& rList,
return rList.Count();
}
-void SwSetExpFieldType::SetChapter( SwSetExpField& rField, const SwNode& rNd )
+void SwSetExpFieldType::SetChapter(SwSetExpField& rField, const SwNode& rNd,
+ SwRootFrame const*const pLayout)
{
- const SwTextNode* pTextNd = rNd.FindOutlineNodeOfLevel( m_nLevel );
+ const SwTextNode* pTextNd = rNd.FindOutlineNodeOfLevel(m_nLevel, pLayout);
if( pTextNd )
{
SwNumRule * pRule = pTextNd->GetNumRule();
@@ -615,19 +651,17 @@ void SwSetExpFieldType::SetChapter( SwSetExpField& rField, const SwNode& rNd )
if (pRule)
{
// --> OD 2005-11-02 #i51089 - TUNING#
- if ( pTextNd->GetNum() )
+ if (SwNodeNum const*const pNum = pTextNd->GetNum(pLayout))
{
- const SwNodeNum & aNum = *(pTextNd->GetNum());
-
// only get the number, without pre-/post-fixstrings
- OUString sNumber( pRule->MakeNumString(aNum, false ));
+ OUString const sNumber(pRule->MakeNumString(*pNum, false));
if( !sNumber.isEmpty() )
- rField.ChgExpStr( sNumber + m_sDelim + rField.GetExpStr() );
+ rField.ChgExpStr(sNumber + m_sDelim + rField.GetExpStr(pLayout), pLayout);
}
else
{
- OSL_FAIL( "<SwSetExpFieldType::SetChapter(..)> - text node with numbering rule, but without number. This is a serious defect" );
+ OSL_ENSURE(pTextNd->GetNum(nullptr), "<SwSetExpFieldType::SetChapter(..)> - text node with numbering rule, but without number. This is a serious defect");
}
}
}
@@ -774,8 +808,10 @@ bool SwSeqFieldList::SeekEntry( const SeqFieldLstElem& rNew, size_t* pP ) const
SwSetExpField::SwSetExpField(SwSetExpFieldType* pTyp, const OUString& rFormel,
sal_uLong nFormat)
- : SwFormulaField( pTyp, nFormat, 0.0 ), mnSeqNo( USHRT_MAX ),
- mnSubType(0)
+ : SwFormulaField( pTyp, nFormat, 0.0 )
+ , m_fValueRLHidden(0.0)
+ , mnSeqNo( USHRT_MAX )
+ , mnSubType(0)
, mpFormatField(nullptr)
{
SetFormula(rFormel);
@@ -784,6 +820,7 @@ SwSetExpField::SwSetExpField(SwSetExpFieldType* pTyp, const OUString& rFormel,
if( IsSequenceField() )
{
SwValueField::SetValue(1.0);
+ m_fValueRLHidden = 1.0;
if( rFormel.isEmpty() )
{
SetFormula(pTyp->GetName() + "+1");
@@ -796,7 +833,7 @@ void SwSetExpField::SetFormatField(SwFormatField & rFormatField)
mpFormatField = &rFormatField;
}
-OUString SwSetExpField::ExpandImpl(SwRootFrame const*const) const
+OUString SwSetExpField::ExpandImpl(SwRootFrame const*const pLayout) const
{
if (mnSubType & nsSwExtendedSubType::SUB_CMD)
{ // we need the CommandString
@@ -804,7 +841,7 @@ OUString SwSetExpField::ExpandImpl(SwRootFrame const*const) const
}
if(!(mnSubType & nsSwExtendedSubType::SUB_INVISIBLE))
{ // value is visible
- return msExpand;
+ return (pLayout && pLayout->IsHideRedlines()) ? msExpandRLHidden : msExpand;
}
return OUString();
}
@@ -836,7 +873,9 @@ std::unique_ptr<SwField> SwSetExpField::Copy() const
std::unique_ptr<SwSetExpField> pTmp(new SwSetExpField(static_cast<SwSetExpFieldType*>(GetTyp()),
GetFormula(), GetFormat()));
pTmp->SwValueField::SetValue(GetValue());
+ pTmp->m_fValueRLHidden = m_fValueRLHidden;
pTmp->msExpand = msExpand;
+ pTmp->msExpandRLHidden = msExpandRLHidden;
pTmp->SetAutomaticLanguage(IsAutomaticLanguage());
pTmp->SetLanguage(GetLanguage());
pTmp->maPText = maPText;
@@ -871,6 +910,32 @@ void SwSetExpField::SetValue( const double& rAny )
GetFormat(), GetLanguage());
}
+void SwSetExpField::SetValue(const double& rValue, SwRootFrame const*const pLayout)
+{
+ if (!pLayout || !pLayout->IsHideRedlines())
+ {
+ SetValue(rValue);
+ }
+ if (!pLayout || pLayout->IsHideRedlines())
+ {
+ m_fValueRLHidden = rValue;
+ if (IsSequenceField())
+ {
+ msExpandRLHidden = FormatNumber(rValue, static_cast<SvxNumType>(GetFormat()), GetLanguage());
+ }
+ else
+ {
+ msExpandRLHidden = static_cast<SwValueFieldType*>(GetTyp())->ExpandValue(
+ rValue, GetFormat(), GetLanguage());
+ }
+ }
+}
+
+double SwSetExpField::GetValue(SwRootFrame const* pLayout) const
+{
+ return (pLayout && pLayout->IsHideRedlines()) ? m_fValueRLHidden : GetValue();
+}
+
void SwGetExpField::SetValue( const double& rAny )
{
SwValueField::SetValue(rAny);
@@ -878,6 +943,25 @@ void SwGetExpField::SetValue( const double& rAny )
GetLanguage());
}
+void SwGetExpField::SetValue(const double& rValue, SwRootFrame const*const pLayout)
+{
+ if (!pLayout || !pLayout->IsHideRedlines())
+ {
+ SetValue(rValue);
+ }
+ if (!pLayout || pLayout->IsHideRedlines())
+ {
+ m_fValueRLHidden = rValue;
+ m_sExpandRLHidden = static_cast<SwValueFieldType*>(GetTyp())->ExpandValue(
+ rValue, GetFormat(), GetLanguage());
+ }
+}
+
+double SwGetExpField::GetValue(SwRootFrame const* pLayout) const
+{
+ return (pLayout && pLayout->IsHideRedlines()) ? m_fValueRLHidden : GetValue();
+}
+
/** Find the index of the reference text following the current field
*
* @param rFormat
@@ -1021,6 +1105,7 @@ bool SwSetExpField::PutValue( const uno::Any& rAny, sal_uInt16 nWhichId )
double fVal = 0.0;
rAny >>= fVal;
SetValue(fVal);
+ m_fValueRLHidden = fVal;
}
break;
case FIELD_PROP_SUBTYPE:
@@ -1044,7 +1129,7 @@ bool SwSetExpField::PutValue( const uno::Any& rAny, sal_uInt16 nWhichId )
{
OUString sTmp;
rAny >>= sTmp;
- ChgExpStr( sTmp );
+ ChgExpStr(sTmp, nullptr);
}
break;
default:
@@ -1101,7 +1186,7 @@ bool SwSetExpField::QueryValue( uno::Any& rAny, sal_uInt16 nWhichId ) const
rAny <<= GetInputFlag();
break;
case FIELD_PROP_PAR4:
- rAny <<= GetExpStr();
+ rAny <<= GetExpStr(nullptr);
break;
default:
return SwField::QueryValue(rAny, nWhichId);
diff --git a/sw/source/core/inc/DocumentFieldsManager.hxx b/sw/source/core/inc/DocumentFieldsManager.hxx
index 0475eb345492..c73345f669c5 100644
--- a/sw/source/core/inc/DocumentFieldsManager.hxx
+++ b/sw/source/core/inc/DocumentFieldsManager.hxx
@@ -56,8 +56,8 @@ public:
virtual bool SetFieldsDirty(bool b, const SwNode* pChk, sal_uLong nLen) override;
virtual void SetFixFields(const DateTime* pNewDateTime) override;
virtual void FieldsToCalc(SwCalc& rCalc, sal_uLong nLastNd, sal_uInt16 nLastCnt) override;
- virtual void FieldsToCalc(SwCalc& rCalc, const SetGetExpField& rToThisField) override;
- virtual void FieldsToExpand(SwHashTable<HashStr>& rTable, const SetGetExpField& rToThisField) override;
+ virtual void FieldsToCalc(SwCalc& rCalc, const SetGetExpField& rToThisField, SwRootFrame const* pLayout) override;
+ virtual void FieldsToExpand(SwHashTable<HashStr>& rTable, const SetGetExpField& rToThisField, SwRootFrame const& rLayout) override;
virtual bool IsNewFieldLst() const override;
virtual void SetNewFieldLst( bool bFlag) override;
virtual void InsDelFieldInFieldLst(bool bIns, const SwTextField& rField) override;
@@ -95,6 +95,8 @@ private:
DocumentFieldsManager(DocumentFieldsManager const&) = delete;
DocumentFieldsManager& operator=(DocumentFieldsManager const&) = delete;
+ void UpdateExpFieldsImpl(SwTextField* pField, SwRootFrame const* pLayout);
+
SwDoc& m_rDoc;
bool mbNewFieldLst; //< TRUE: Rebuild field-list.
diff --git a/sw/source/core/unocore/unofield.cxx b/sw/source/core/unocore/unofield.cxx
index b8ada47916cd..4fe387ba3b59 100644
--- a/sw/source/core/unocore/unofield.cxx
+++ b/sw/source/core/unocore/unofield.cxx
@@ -1730,7 +1730,7 @@ void SAL_CALL SwXTextField::attach(
pSEField->SetInputFlag(m_pImpl->m_pProps->bBool1);
pSEField->SetPromptText(m_pImpl->m_pProps->sPar3);
if (!m_pImpl->m_pProps->sPar4.isEmpty())
- pSEField->ChgExpStr(m_pImpl->m_pProps->sPar4);
+ pSEField->ChgExpStr(m_pImpl->m_pProps->sPar4, nullptr);
}
break;
@@ -1772,7 +1772,7 @@ void SAL_CALL SwXTextField::attach(
xField.reset(pGEField);
//TODO: evaluate SubType!
if (!m_pImpl->m_pProps->sPar4.isEmpty())
- pGEField->ChgExpStr(m_pImpl->m_pProps->sPar4);
+ pGEField->ChgExpStr(m_pImpl->m_pProps->sPar4, nullptr);
// #i82544#
if (bSetGetExpFieldUninitialized)
pGEField->SetLateInitialization();
diff --git a/sw/source/filter/ww8/ww8par5.cxx b/sw/source/filter/ww8/ww8par5.cxx
index 51f6ca542053..49c506b3d5d6 100644
--- a/sw/source/filter/ww8/ww8par5.cxx
+++ b/sw/source/filter/ww8/ww8par5.cxx
@@ -1393,7 +1393,7 @@ eF_ResT SwWW8ImplReader::Read_F_ANumber( WW8FieldDesc*, OUString& rStr )
}
SwSetExpField aField( static_cast<SwSetExpFieldType*>(m_pNumFieldType), OUString(),
GetNumberPara( rStr ) );
- aField.SetValue( ++m_nFieldNum );
+ aField.SetValue( ++m_nFieldNum, nullptr );
m_rDoc.getIDocumentContentOperations().InsertPoolItem( *m_pPaM, SwFormatField( aField ) );
return eF_ResT::OK;
}
commit 9c1415d844409a1ef03f048da77e8dfd165a321d
Author: Michael Stahl <Michael.Stahl at cib.de>
AuthorDate: Mon Nov 12 14:39:08 2018 +0100
Commit: Michael Stahl <Michael.Stahl at cib.de>
CommitDate: Tue Nov 13 13:55:52 2018 +0100
sw: booleans are hard, lets go shopping
Change-Id: Iefa49407de90bace5eea2f86170ba8cd29b1cfe7
diff --git a/sw/source/core/doc/DocumentFieldsManager.cxx b/sw/source/core/doc/DocumentFieldsManager.cxx
index 0a3a8a47a8c3..f24fd9688238 100644
--- a/sw/source/core/doc/DocumentFieldsManager.cxx
+++ b/sw/source/core/doc/DocumentFieldsManager.cxx
@@ -1578,7 +1578,7 @@ void DocumentFieldsManager::SetNewFieldLst(bool bFlag)
void DocumentFieldsManager::InsDelFieldInFieldLst( bool bIns, const SwTextField& rField )
{
- if( !mbNewFieldLst || !m_rDoc.IsInDtor() )
+ if (!mbNewFieldLst && !m_rDoc.IsInDtor())
mpUpdateFields->InsDelFieldInFieldLst( bIns, rField );
}
commit d0f285c147de49007284754fc0b0e27e62343bd7
Author: Michael Stahl <Michael.Stahl at cib.de>
AuthorDate: Mon Nov 12 13:36:49 2018 +0100
Commit: Michael Stahl <Michael.Stahl at cib.de>
CommitDate: Tue Nov 13 13:55:52 2018 +0100
sw: flag check looks incorrect in SwDocUpdateField::MakeFieldList()
Change-Id: Ifebbaa6ef21ac518f62a726dadaceb97fd5e446b
diff --git a/sw/source/core/doc/docfld.cxx b/sw/source/core/doc/docfld.cxx
index 4a7a80d23129..42d47a4cc0e6 100644
--- a/sw/source/core/doc/docfld.cxx
+++ b/sw/source/core/doc/docfld.cxx
@@ -801,7 +801,8 @@ void SwDocUpdateField::InsDelFieldInFieldLst( bool bIns, const SwTextField& rFie
void SwDocUpdateField::MakeFieldList( SwDoc& rDoc, bool bAll, int eGetMode )
{
- if (!m_pFieldSortList || bAll || !(eGetMode & m_nFieldListGetMode)
+ if (!m_pFieldSortList || bAll
+ || ((eGetMode & m_nFieldListGetMode) != eGetMode)
|| rDoc.GetNodes().Count() != m_nNodes)
{
MakeFieldList_( rDoc, eGetMode );
commit 8e548469fd63b1ef8a71966385c20b2146494692
Author: Michael Stahl <Michael.Stahl at cib.de>
AuthorDate: Mon Nov 12 13:23:20 2018 +0100
Commit: Michael Stahl <Michael.Stahl at cib.de>
CommitDate: Tue Nov 13 13:55:52 2018 +0100
sw: prefix members of SwDocUpdateField & trivial cleanups
Change-Id: Ied1e89279e3071e0c6c1b414bb23044e6439258c
diff --git a/sw/source/core/doc/DocumentFieldsManager.cxx b/sw/source/core/doc/DocumentFieldsManager.cxx
index efb42e357632..0a3a8a47a8c3 100644
--- a/sw/source/core/doc/DocumentFieldsManager.cxx
+++ b/sw/source/core/doc/DocumentFieldsManager.cxx
@@ -146,7 +146,7 @@ namespace sw
DocumentFieldsManager::DocumentFieldsManager( SwDoc& i_rSwdoc ) : m_rDoc( i_rSwdoc ),
mbNewFieldLst(true),
- mpUpdateFields( new SwDocUpdateField( &m_rDoc ) ),
+ mpUpdateFields(new SwDocUpdateField(m_rDoc)),
mpFieldTypes( new SwFieldTypes ),
mnLockExpField( 0 )
{
@@ -849,7 +849,7 @@ void DocumentFieldsManager::UpdateExpFields( SwTextField* pUpdateField, bool bUp
mpUpdateFields->MakeFieldList( m_rDoc, true, GETFLD_ALL );
mbNewFieldLst = false;
- if( mpUpdateFields->GetSortLst()->empty() )
+ if (mpUpdateFields->GetSortList()->empty())
{
if( bUpdRefFields )
UpdateRefFields();
@@ -927,9 +927,9 @@ void DocumentFieldsManager::UpdateExpFields( SwTextField* pUpdateField, bool bUp
std::unordered_map<SwSetExpFieldType const*, SwTextNode const*> SetExpOutlineNodeMap;
- for( SetGetExpFields::const_iterator it = mpUpdateFields->GetSortLst()->begin(); it != mpUpdateFields->GetSortLst()->end(); ++it )
+ for (std::unique_ptr<SetGetExpField> const& it : *mpUpdateFields->GetSortList())
{
- SwSection* pSect = const_cast<SwSection*>((*it)->GetSection());
+ SwSection* pSect = const_cast<SwSection*>(it->GetSection());
if( pSect )
{
SwSbxValue aValue = aCalc.Calculate(
@@ -960,7 +960,7 @@ void DocumentFieldsManager::UpdateExpFields( SwTextField* pUpdateField, bool bUp
continue;
}
- SwTextField* pTextField = const_cast<SwTextField*>((*it)->GetTextField());
+ SwTextField* pTextField = const_cast<SwTextField*>(it->GetTextField());
if( !pTextField )
{
OSL_ENSURE( false, "what's wrong now'" );
@@ -1126,7 +1126,7 @@ void DocumentFieldsManager::UpdateExpFields( SwTextField* pUpdateField, bool bUp
if( MAXLEVEL > nLvl )
{
// test if the Number needs to be updated
- pSeqNd = m_rDoc.GetNodes()[ (*it)->GetNode() ];
+ pSeqNd = m_rDoc.GetNodes()[ it->GetNode() ];
const SwTextNode* pOutlNd = pSeqNd->
FindOutlineNodeOfLevel( nLvl );
@@ -1211,13 +1211,12 @@ sal_Int32 DocumentFieldsManager::GetRecordsPerDocument() const
sal_Int32 nRecords = 1;
mpUpdateFields->MakeFieldList( m_rDoc, true, GETFLD_ALL );
- if( mpUpdateFields->GetSortLst()->empty() )
+ if (mpUpdateFields->GetSortList()->empty())
return nRecords;
- for( SetGetExpFields::const_iterator it = mpUpdateFields->GetSortLst()->begin();
- it != mpUpdateFields->GetSortLst()->end(); ++it )
+ for (std::unique_ptr<SetGetExpField> const& it : *mpUpdateFields->GetSortList())
{
- const SwTextField *pTextField = (*it)->GetTextField();
+ const SwTextField *pTextField = it->GetTextField();
if( !pTextField )
continue;
@@ -1443,13 +1442,15 @@ void DocumentFieldsManager::FieldsToCalc( SwCalc& rCalc, const SetGetExpField& r
pMgr->CloseAll(false);
#endif
- if( !mpUpdateFields->GetSortLst()->empty() )
+ if (!mpUpdateFields->GetSortList()->empty())
{
SetGetExpFields::const_iterator const itLast =
- mpUpdateFields->GetSortLst()->upper_bound(
+ mpUpdateFields->GetSortList()->upper_bound(
&rToThisField);
- for( SetGetExpFields::const_iterator it = mpUpdateFields->GetSortLst()->begin(); it != itLast; ++it )
+ for (auto it = mpUpdateFields->GetSortList()->begin(); it != itLast; ++it)
+ {
lcl_CalcField( m_rDoc, rCalc, **it, pMgr );
+ }
}
#if HAVE_FEATURE_DBCONNECTIVITY
pMgr->CloseAll(false);
@@ -1469,8 +1470,8 @@ void DocumentFieldsManager::FieldsToCalc( SwCalc& rCalc, sal_uLong nLastNd, sal_
pMgr->CloseAll(false);
#endif
- for( SetGetExpFields::const_iterator it = mpUpdateFields->GetSortLst()->begin();
- it != mpUpdateFields->GetSortLst()->end() &&
+ for(auto it = mpUpdateFields->GetSortList()->begin();
+ it != mpUpdateFields->GetSortList()->end() &&
( (*it)->GetNode() < nLastNd ||
( (*it)->GetNode() == nLastNd && (*it)->GetContent() <= nLastCnt )
);
@@ -1493,13 +1494,13 @@ void DocumentFieldsManager::FieldsToExpand( SwHashTable<HashStr> & rHashTable,
// Hash table for all string replacements is filled on-the-fly.
// Try to fabricate an uneven number.
- sal_uInt16 nTableSize = (( mpUpdateFields->GetSortLst()->size() / 7 ) + 1 ) * 7;
+ sal_uInt16 nTableSize = ((mpUpdateFields->GetSortList()->size() / 7) + 1) * 7;
rHashTable.resize(nTableSize);
SetGetExpFields::const_iterator const itLast =
- mpUpdateFields->GetSortLst()->upper_bound(&rToThisField);
+ mpUpdateFields->GetSortList()->upper_bound(&rToThisField);
- for( SetGetExpFields::const_iterator it = mpUpdateFields->GetSortLst()->begin(); it != itLast; ++it )
+ for (auto it = mpUpdateFields->GetSortList()->begin(); it != itLast; ++it)
{
const SwTextField* pTextField = (*it)->GetTextField();
if( !pTextField )
diff --git a/sw/source/core/inc/docfld.hxx b/sw/source/core/inc/docfld.hxx
index 17704c22448b..c9d5fba43048 100644
--- a/sw/source/core/inc/docfld.hxx
+++ b/sw/source/core/inc/docfld.hxx
@@ -131,25 +131,25 @@ const int GETFLD_EXPAND = 2;
class SwDocUpdateField
{
- std::unique_ptr<SetGetExpFields> pFieldSortLst; // current field list for calculation
- SwHashTable<SwCalcFieldType> aFieldTypeTable;
+ std::unique_ptr<SetGetExpFields> m_pFieldSortList; ///< current field list for calculation
+ SwHashTable<SwCalcFieldType> m_FieldTypeTable;
- sal_uLong nNodes; // if the node count is different
- sal_uInt8 nFieldLstGetMode;
- SwDoc* pDocument;
+ sal_uLong m_nNodes; ///< to check if the node count changed
+ int m_nFieldListGetMode;
+ SwDoc& m_rDoc;
- bool bInUpdateFields : 1; // currently there is an UpdateFields
- bool bFieldsDirty : 1; // some fields are invalid
+ bool m_bInUpdateFields : 1; ///< currently in an UpdateFields call
+ bool m_bFieldsDirty : 1; ///< some fields are invalid
void MakeFieldList_( SwDoc& pDoc, int eGetMode );
void GetBodyNode( const SwTextField& , SwFieldIds nFieldWhich );
void GetBodyNode( const SwSectionNode&);
public:
- SwDocUpdateField(SwDoc* pDocument);
+ SwDocUpdateField(SwDoc& rDocument);
~SwDocUpdateField();
- const SetGetExpFields* GetSortLst() const { return pFieldSortLst.get(); }
+ const SetGetExpFields* GetSortList() const { return m_pFieldSortList.get(); }
void MakeFieldList( SwDoc& rDoc, bool bAll, int eGetMode );
@@ -158,21 +158,21 @@ public:
void InsertFieldType( const SwFieldType& rType );
void RemoveFieldType( const SwFieldType& rType );
- bool IsInUpdateFields() const { return bInUpdateFields; }
- void SetInUpdateFields( bool b ) { bInUpdateFields = b; }
+ bool IsInUpdateFields() const { return m_bInUpdateFields; }
+ void SetInUpdateFields( bool b ) { m_bInUpdateFields = b; }
- bool IsFieldsDirty() const { return bFieldsDirty; }
+ bool IsFieldsDirty() const { return m_bFieldsDirty; }
void SetFieldsDirty( bool b )
{
- bFieldsDirty = b;
+ m_bFieldsDirty = b;
if (b)
{
- pDocument->getIDocumentTimerAccess().StartIdling();
+ m_rDoc.getIDocumentTimerAccess().StartIdling();
}
}
- SwHashTable<SwCalcFieldType> const & GetFieldTypeTable() const { return aFieldTypeTable; }
+ SwHashTable<SwCalcFieldType> const& GetFieldTypeTable() const { return m_FieldTypeTable; }
};
#endif
More information about the Libreoffice-commits
mailing list
diff --git a/sw/source/core/doc/docfld.cxx b/sw/source/core/doc/docfld.cxx
index 3affc7cdd901..4a7a80d23129 100644
--- a/sw/source/core/doc/docfld.cxx
+++ b/sw/source/core/doc/docfld.cxx
@@ -775,11 +775,11 @@ void SwDocUpdateField::InsDelFieldInFieldLst( bool bIns, const SwTextField& rFie
}
SetFieldsDirty( true );
- if( !pFieldSortLst )
+ if (!m_pFieldSortList)
{
if( !bIns ) // if list is present and deleted
return; // don't do a thing
- pFieldSortLst.reset(new SetGetExpFields);
+ m_pFieldSortList.reset(new SetGetExpFields);
}
if( bIns ) // insert anew:
@@ -788,26 +788,30 @@ void SwDocUpdateField::InsDelFieldInFieldLst( bool bIns, const SwTextField& rFie
{
// look up via the pTextField pointer. It is a sorted list, but it's sorted by node
// position. Until this is found, the search for the pointer is already done.
- for( SetGetExpFields::size_type n = 0; n < pFieldSortLst->size(); ++n )
- if( &rField == (*pFieldSortLst)[ n ]->GetPointer() )
+ for (SetGetExpFields::size_type n = 0; n < m_pFieldSortList->size(); ++n)
+ {
+ if (&rField == (*m_pFieldSortList)[n]->GetPointer())
{
- pFieldSortLst->erase(n);
+ m_pFieldSortList->erase(n);
n--; // one field can occur multiple times
}
+ }
}
}
void SwDocUpdateField::MakeFieldList( SwDoc& rDoc, bool bAll, int eGetMode )
{
- if( !pFieldSortLst || bAll || !( eGetMode & nFieldLstGetMode ) ||
- rDoc.GetNodes().Count() != nNodes )
+ if (!m_pFieldSortList || bAll || !(eGetMode & m_nFieldListGetMode)
+ || rDoc.GetNodes().Count() != m_nNodes)
+ {
MakeFieldList_( rDoc, eGetMode );
+ }
}
void SwDocUpdateField::MakeFieldList_( SwDoc& rDoc, int eGetMode )
{
// new version: walk all fields of the attribute pool
- pFieldSortLst.reset(new SetGetExpFields);
+ m_pFieldSortList.reset(new SetGetExpFields);
// consider and unhide sections
// with hide condition, only in mode GETFLD_ALL (<eGetMode == GETFLD_ALL>)
@@ -820,7 +824,7 @@ void SwDocUpdateField::MakeFieldList_( SwDoc& rDoc, int eGetMode )
// the hide conditions of section have to be updated.
// For correct updating the hide condition of a section, its position
// have to be known in order to insert the hide condition as a new
// expression field into the sorted field list (<pFieldSortLst>).
if ( eGetMode == GETFLD_ALL )
// Collect the sections first. Supply sections that are hidden by condition
// with frames so that the contained fields are sorted properly.
@@ -985,8 +989,8 @@ void SwDocUpdateField::MakeFieldList_( SwDoc& rDoc, int eGetMode )
GetBodyNode( *pTextField, nWhich );
}
}
- nFieldLstGetMode = static_cast<sal_uInt8>( eGetMode );
- nNodes = rDoc.GetNodes().Count();
+ m_nFieldListGetMode = eGetMode;
+ m_nNodes = rDoc.GetNodes().Count();
}
void SwDocUpdateField::GetBodyNode( const SwTextField& rTField, SwFieldIds nFieldWhich )
@@ -1039,7 +1043,7 @@ void SwDocUpdateField::GetBodyNode( const SwTextField& rTField, SwFieldIds nFiel
}
#endif
if( pNew != nullptr )
- pFieldSortLst->insert( std::move(pNew) );
+ m_pFieldSortList->insert( std::move(pNew) );
}
void SwDocUpdateField::GetBodyNode( const SwSectionNode& rSectNd )
@@ -1078,7 +1082,7 @@ void SwDocUpdateField::GetBodyNode( const SwSectionNode& rSectNd )
if( !pNew )
pNew.reset(new SetGetExpField( rSectNd ));
- pFieldSortLst->insert( std::move(pNew) );
+ m_pFieldSortList->insert( std::move(pNew) );
}
void SwDocUpdateField::InsertFieldType( const SwFieldType& rType )
@@ -1108,8 +1112,8 @@ void SwDocUpdateField::InsertFieldType( const SwFieldType& rType )
if( !pFnd )
{
SwCalcFieldType* pNew = new SwCalcFieldType( sFieldName, &rType );
- pNew->pNext.reset( aFieldTypeTable[ n ].release() );
- aFieldTypeTable[ n ].reset(pNew);
+ pNew->pNext.reset( m_FieldTypeTable[n].release() );
+ m_FieldTypeTable[n].reset(pNew);
}
}
}
@@ -1138,13 +1142,13 @@ void SwDocUpdateField::RemoveFieldType( const SwFieldType& rType )
SwCalcFieldType* pFnd = GetFieldTypeTable().Find( sFieldName, &n );
if( pFnd )
{
- if( aFieldTypeTable[ n ].get() == pFnd )
+ if (m_FieldTypeTable[n].get() == pFnd)
{
- aFieldTypeTable[ n ].reset(static_cast<SwCalcFieldType*>(pFnd->pNext.release()));
+ m_FieldTypeTable[n].reset(static_cast<SwCalcFieldType*>(pFnd->pNext.release()));
}
else
{
- SwHash* pPrev = aFieldTypeTable[ n ].get();
+ SwHash* pPrev = m_FieldTypeTable[n].get();
while( pPrev->pNext.get() != pFnd )
pPrev = pPrev->pNext.get();
pPrev->pNext = std::move(pFnd->pNext);
@@ -1154,13 +1158,13 @@ void SwDocUpdateField::RemoveFieldType( const SwFieldType& rType )
}
}
-SwDocUpdateField::SwDocUpdateField(SwDoc* pDoc)
- : aFieldTypeTable(TBLSZ)
- , nNodes(0)
- , nFieldLstGetMode(0)
- , pDocument(pDoc)
- , bInUpdateFields(false)
- , bFieldsDirty(false)
+SwDocUpdateField::SwDocUpdateField(SwDoc& rDoc)
+ : m_FieldTypeTable(TBLSZ)
+ , m_nNodes(0)
+ , m_nFieldListGetMode(0)
+ , m_rDoc(rDoc)
+ , m_bInUpdateFields(false)
+ , m_bFieldsDirty(false)
{
}
@@ -820,7 +824,7 @@ void SwDocUpdateField::MakeFieldList_( SwDoc& rDoc, int eGetMode )
// For correct updating the hide condition of a section, its position
// have to be known in order to insert the hide condition as a new
- // expression field into the sorted field list (<pFieldSortLst>).
+ // expression field into the sorted field list (<m_pFieldSortList>).
if ( eGetMode == GETFLD_ALL )
// Collect the sections first. Supply sections that are hidden by condition